@@ -1,5 +1,5 @@
 #!/bin/bash
-# $Id: c31e15aab2646e71c467f945182ced082199e3eb $
+# $Id: 70a49695d9298f7e4077ebb9e8b9f7390bb5dec6 $
 
 isTrue() {
 	case "$1" in
@@ -2054,6 +2054,40 @@ expand_file() {
 	echo "${expanded_file}"
 }
 
+find_and_unpack() {
+	local flist
+
+	local fmt
+	for fmt in "$@"
+	do
+		case "${fmt}" in
+		"gz"|"xz"|"zstd")
+			flist=( $(find -type f -name "*.${fmt}") )
+			;;
+		*)
+			gen_die "unknown compression format: ${fmt}"
+			;;
+		esac
+
+		if [ ${#flist[@]} -lt 1 ]
+		then
+			continue
+		fi
+
+		case "${fmt}" in
+		"gz")
+			gunzip "${flist[@]}"
+			;;
+		"xz")
+			unxz "${flist[@]}"
+			;;
+		"zstd")
+			unzstd "${flist[@]}"
+			;;
+		esac
+	done
+}
+
 find_kernel_binary() {
 	local kernel_binary=${*}
 	local kernel_binary_found=
