@@ -1,5 +1,5 @@
 #!/bin/bash
-# $Id: 2cfa0fdbb7dbd9a8ec1d91e6b7991c310ae3afa8 $
+# $Id: 87e514940b38628c67587d6a4100e92fee8f069a $
 
 COPY_BINARIES=false
 CPIO_ARGS="--quiet --null -o -H newc --owner root:root --force-local"
@@ -1773,6 +1773,7 @@ append_firmware() {
 		fwlist=( "${FIRMWARE_FILES[@]}" )
 	else
 		local myfw=
+		local -a myfw_f=()
 		while IFS= read -r -u 3 myfw
 		do
 			if [ -z "${myfw}" ]
@@ -1780,13 +1781,20 @@ append_firmware() {
 				gen_die "modinfo error!"
 			fi
 
-			if [ ! -f "${FIRMWARE_DIR}/${myfw}" ]
+			myfw_f=( $(compgen -G "${FIRMWARE_DIR}/${myfw}*") )
+
+			if [ ${#myfw_f[@]} -gt 1 ]
+			then
+				gen_die "excessive number of firmwares!"
+			fi
+
+			if [ ${#myfw_f[@]} -lt 1 ]
 			then
 				print_warning 3 "$(get_indent 3) - ${myfw} is missing; Ignoring ..."
 				continue
 			fi
 
-			fwlist+=( "${myfw}" )
+			fwlist+=( "${myfw_f#${FIRMWARE_DIR}/}" )
 		done 3< <( (
 			modinfo -b "${KERNEL_MODULES_PREFIX%/}" -k "${KV}" -F firmware $(mod_dep_list) 2>/dev/null || echo
 		) | sort | uniq )
@@ -1798,6 +1806,10 @@ append_firmware() {
 		cp -rL --parents --target-directory="${TDIR}/lib/firmware" "${fwlist[@]}" 2>/dev/null \
 			|| gen_die "Failed to copy firmware files to '${TDIR}/lib/firmware'!"
 		popd &>/dev/null || gen_die "Failed to chdir!"
+
+		pushd "${TDIR}/lib/firmware" &>/dev/null || gen_die "Failed to chdir to '${TDIR}/lib/firmware'!"
+		find_and_unpack xz zstd
+		popd &>/dev/null || gen_die "Failed to chdir!"
 	fi
 
 	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
@@ -1911,7 +1923,7 @@ append_modules() {
 
 		if [ ! -f "${mymod}" ]
 		then
-			gen_die "Module '${i}${KEXT}' is missing!"
+			gen_die "Module '${mymod}' is missing!"
 		fi
 
 		modlist+=( "${mymod/#${modules_srcdir}\//}" )
@@ -1932,6 +1944,10 @@ append_modules() {
 	cp -ax --parents --target-directory "${modules_dstdir}" modules* 2>/dev/null \
 		|| gen_die "Failed to copy '${modules_srcdir}/modules*' to '${modules_dstdir}'!"
 
+	pushd "${modules_dstdir}" &>/dev/null || gen_die "Failed to chdir to '${modules_dstdir}'!"
+	find_and_unpack gz xz zstd
+	popd &>/dev/null || gen_die "Failed to chdir!"
+
 	print_info 2 "$(get_indent 2)modules: Updating modules.dep ..."
 	local depmod_cmd=( depmod -a -b "${TDIR}" ${KV} )
 	print_info 3 "COMMAND: ${depmod_cmd[*]}" 1 0 1
